function rFEM = FEspace(FEM)

info_start(FEM,'FE space');

rFEM = FEM;
Th = FEM.Th;
Ref = FEM.Ref;

% gather grid information
ne = Th.M;
Vh.M = ne;
nc = Th.nc;
% mapping from reference to actual node numbers
elementnodes = zeros(ne,Ref.nnodes);
% number of interior nodes per element
ni = Ref.nodedis(2);
% initial node numbers
inds = (nc+1):(nc+ni);
inodes = zeros(ne*ni,2);
% internal node counter
indcnt = 1;
% node-on-edge counter
totnds = 0;
% generate Edgelist: Edgelist(i,j) yields edge number
Edgelist = sparse(nc,nc);
% use Euler's formula to compute number of edges
Edge = cell(ne+nc-1,1);
cnt = 0;
for k = 1:ne
    % get an element
    e = Th.elements(k,:);
    nedges = length(e);
    % remember actual node numbers
    elementnodes(k,Ref.shape) = e;
    % add interior nodes if any
    if ni
        elementnodes(k,Ref.interior) = inds;
        F = Ref.makeF*Th.corners(e,:);
        for j = 1:ni
            inodes(indcnt,:)=[1,Ref.nodes(Ref.interior(j),:)]*F;
            indcnt = indcnt +1;
        end
        inds = inds+ni;
    end
    % loop over its edges
    e(end+1)=e(1);
    for j = 1:nedges
        ind = sort([e(j),e(j+1)]);
        enum = Edgelist(ind(1),ind(2));
        if enum
            % edge already exists
            Edge{enum}.n = Edge{enum}.n +1;
            Edge{enum}.element(2) = k;
            Edge{enum}.edge(2)=j;
        else
            % edge does not yet exist
            cnt = cnt +1;
            Edge{cnt}.n = 1;
            Edge{cnt}.ind = ind;
            Edge{cnt}.element(1) = k;
            Edge{cnt}.edge(1)=j;
            Edge{cnt}.nds = Ref.nodedis(2+j);
            totnds = totnds + Edge{cnt}.nds;
            Edgelist(ind(1),ind(2)) = cnt;
        end
    end
end        
Vh.edgelist = Edgelist;
Vh.edge = Edge;
% number of edges
nk = length(Edge);
% counter for edge nodes
encnt = nc+Ref.nodedis(2)*ne;
% coordinates of nodes-on-edge
nodesonedge = zeros(totnds,2);
noecnt = 1; 
% loop over edges to set up element node indices
for k = 1:nk
    % number of points on this edge
    nren = Edge{k}.nds;
    if nren
        % new batch of global node numbers
        ind = encnt+(1:nren);
        % number of first element
        enum = Edge{k}.element(1);
        % reference numbers of nodes on edge
        nref = Ref.onedges{Edge{k}.edge(1)};
        % assign to nodes on edge of first element
        elementnodes(enum,nref) = ind;
        % compute node coordinates
        F = Ref.makeF*Th.corners(Th.elements(enum,:),:);
        for j = 1:length(nref)
            nodesonedge(noecnt,:)=[1,Ref.nodes(nref(j),:)]*F;
            noecnt = noecnt +1;
        end
        if Edge{k}.n>1
            % check whether edge orientation are the same
            if Th.elements(Edge{k}.element(1),Edge{k}.edge(1)) ~= Th.elements(Edge{k}.element(2),Edge{k}.edge(2))
                % opposite orientation
                ind = fliplr(ind);
            end
            elementnodes(Edge{k}.element(2),Ref.onedges{Edge{k}.edge(2)}) = ind;        
        end
        % update total node number
        encnt = encnt + nren;
    end
end
Vh.elementnodes = elementnodes;
% prepare space for shape and interior node coordinates
Vh.nodes = [Th.corners ; inodes ; nodesonedge];
nds = size(Vh.nodes,1);
Vh.nnodes = nds;
% 
selct = true(nds,1);
for k = 1:size(Th.Dirichlet,1)
    % find edge number
    edg = Th.Dirichlet(k,:);
    edg = [min(edg),max(edg)];
    enum = Edgelist(edg(1),edg(2));
    e = Edge{enum};
    nods = [edg , elementnodes(e.element,Ref.onedges{e.edge})];
    selct(nods)=false;
end
Vh.Dirinodes = find(~selct);
% dof to node map
Vh.doftonode = (1:nds)';
Vh.doftonode = Vh.doftonode(selct);
Vh.ndof = length(Vh.doftonode); 
% inverse map
Vh.nodetodof = zeros(nds,1);
Vh.nodetodof(Vh.doftonode) = 1:Vh.ndof;

rFEM.Vh = Vh;

info_end(FEM);
