function info(varargin)

if nargin
    FEM = varargin{1};
end
fprintf('\n\ninfo on FEM data structure:\n');
fprintf('---------------------------\n');
fprintf(' FEM.parameters: problem specific data\n\n');
fprintf(' FEM.Ref: description of reference element\n');
fprintf('## node management\n');
fprintf('<nnodes> number of nodes in element');
if nargin
    fprintf(' (%d)',FEM.Ref.nnodes);
end
fprintf('\n');
fprintf('<nodes> node coordinates [nnodes x dim]');
fprintf('\n');
fprintf('<shape> node numbers of element vertices [N x 1]');
fprintf('\n');
fprintf('<interior> node numbers of interior nodes [N x 1]');
fprintf('\n');
fprintf('<onedges> node numbers on element edges {[N1 x 1] , ... , [Nk x 1]}');
fprintf('\n');
fprintf('<nodedis> number of entries in shape/interior/onedges');
fprintf('\n');
fprintf('<makeF> matrix to set up affine map F to an element\n');
fprintf('        if the element vertex coordinates are given by the row vectors c1,...,cN\n');
fprintf('        then F(x,y) = [1 x y]*Ref.makeF*[c1;c2;...;cN]');
fprintf('\n');
fprintf('## quadrature rules\n');
fprintf('<intx> integration points inside the element [N x dim]');
fprintf('\n');
fprintf('<intw> corresponding integration weights [N x 1]');
fprintf('\n');
fprintf('<bdrx{j}> integration points on j-th element edge [N x 1]');
fprintf('\n');
fprintf('<bdrw{j}> corresponding integration weights [N x 1]');
fprintf('\n');
fprintf('## basis function management\n');
fprintf('<s> degrees of freedom');
if nargin
    fprintf(' (%d)',FEM.Ref.s);
end
fprintf('\n');
fprintf('<basis> function handle to evaluate i-th basis function at points x\n');
fprintf('        basis(i,x) returns k values if x is of dimension [k x dim]');
fprintf('\n');
fprintf('<dbasis{k}> function handle for the j-th gradient component of i-th basis function\n');
fprintf('            dbasis{j}(i,x) returns k values if x is of dimension [k x dim]');
fprintf('\n');
fprintf('<baseval(:,i)> values of the i-th basis function at interior integration nodes');
fprintf('\n');
fprintf('<basevalbdr{k}(:,i)> values of the i-th basis function at integration nodes on edge k');
fprintf('\n');
fprintf('<dbaseval(j,:,i)> values of the j-th gradient component of i-th basis function at interior integration nodes');
fprintf('\n');
fprintf('<dbasevalbdr{k}(j,:,i)> values of the j-th gradient component of the i-th basis function at integration nodes on edge k');
fprintf('\n\n');
fprintf(' FEM.Th: triangulation data\n');
fprintf('<dim> problem dimension');
if nargin
    fprintf(' (%d)',FEM.Th.dim);
end
fprintf('\n');
fprintf('<M> number of elements');
if nargin
    fprintf(' (%d)',FEM.Th.M);
end
fprintf('\n');
fprintf('<nc> number of element vertices');
if nargin
    fprintf(' (%d)',FEM.Th.nc);
end
fprintf('\n');
fprintf('<corners> element vertices [nc x dim]');
fprintf('\n');
fprintf('<elements> connectivity information [M x vert. per element]');
fprintf('\n');
fprintf('<ndoms> number of subdomains');
if nargin
    fprintf(' (%d)',FEM.Th.ndoms);
end
fprintf('\n');
fprintf('<domains{k}> element numbers in subdomain k');
fprintf('\n');
fprintf('<Dirichlet> list of Dirichlet edges [number of edges x 2]');
fprintf('\n\n');
fprintf(' FEM.Vh: information on finite element space\n');
fprintf('<M> number of elements');
if nargin
    fprintf(' (%d)',FEM.Vh.M);
end
fprintf('\n');
fprintf('<nnodes> number of nodes');
if nargin
    fprintf(' (%d)',FEM.Vh.nnodes);
end
fprintf('\n');
fprintf('<ndof> number of degrees of freedom');
if nargin
    fprintf(' (%d)',FEM.Vh.ndof);
end
fprintf('\n');
fprintf('<nodes> coordinates of nodes [nnodes x dim]\n');
fprintf('<elementnodes(k,j)> node number of reference node j in element k\n');
fprintf('<edgelist(i,j)> edge number with element vertices i<j as endpoints\n');
fprintf('<edge{k}> data structure describing k-th edge\n');
fprintf('          <n> number of elements adjacent to the edge\n');
fprintf('          <ind> numbers of element vertices corresponding to the edge\n');
fprintf('          <element> numbers of the adjacent elements\n');
fprintf('          <edge(i)> edge number in i-th adjacent element\n');
fprintf('          <nds> number of points between element vertices\n');
fprintf('<doftonode(k)> node number corresponding to dof k\n');
fprintf('<nodetodof(k)> dof corresponding to node k\n');
fprintf('<Dirinodes> node numbers corresponding to nodes on Dirichlet edges\n');
fprintf('\n');
fprintf(' FEM.matrices: matrices to linear and bilinear forms\n');
fprintf('<Afull> stiffness matrix on H (full space)\n');
fprintf('<A> stiffness matrix restricted to V (problem space with Dirichlet restriction)\n');
fprintf('<Mfull> mass matrix on H\n');
fprintf('<M> stiffness matrix restricted to V\n');
fprintf('<bfull> linear form on H\n');
fprintf('<b> linear form restricted to V\n');
fprintf('\n');
fprintf(' FEM.solution: solution and postprocessing\n');
fprintf('<u> solution coordinates in basis of V\n');
fprintf('<ufull> solution in basis of H\n');
fprintf('<gradient> data structure on gradient of solution\n');
fprintf('           <P> centroid coordinates of elements [M x dim]\n');
fprintf('           <G> solution gradient at centroids [M x dim]\n');
