function rFEM = P1(FEM)
% generate data structure of reference element
    info_start(FEM,'reference element');

    % degrees of freedom
    Ref.s = 3;
    % nodes
    Ref.nodes = [0 , 0;
                 1 , 0;
                 0 , 1];
    N = size(Ref.nodes,1);
    % matrix to set up affine map F to an element with corners 
    % given by the row vectors c1,...,cN
    % F(x,y) = [1 x y]*Ref.makeF*[c1;c2;...;cN]
    Ref.makeF = [1 -1 -1;0 1 0;0 0 1]';
    % nodes defining element shape
    Ref.shape = [1;2;3];
    % nodes in the interior of the element
    Ref.interior = [];
    % nodes on edges of the element
    Ref.onedges = {[],[],[]};
    Ref.nodedis = zeros(length(Ref.onedges)+2);
    Ref.nodedis = [length(Ref.shape), length(Ref.interior)];
    for k = 1:length(Ref.onedges)
        Ref.nodedis(k+2) = length(Ref.onedges{k});
    end
    Ref.nnodes = sum(Ref.nodedis);

    % basis functions corresponding to nodes
    function y = basis(i,x)
        switch i
            case 1
                y = 1 - x(:,1) -x(:,2);
            case 2
                y = x(:,1);
            case 3
                y = x(:,2);
            otherwise
                y = NaN(size(x,1),1);
        end
    end
    % partial derivative of basis functions corresponding to nodes
    function y = d1basis(i,x)
        switch i
            case 1
                y = -ones(size(x,1),1);
            case 2
                y = ones(size(x,1),1);
            case 3
                y = zeros(size(x,1),1);
            otherwise
                y = NaN(size(x,1),1);
        end
    end
    % partial derivative of basis functions corresponding to nodes
    function y = d2basis(i,x)
        switch i
            case 1
                y = -ones(size(x,1),1);
            case 2
                y = zeros(size(x,1),1);
            case 3
                y = ones(size(x,1),1);
            otherwise
                y = NaN(size(x,1),1);
        end
    end
    Ref.basis = @basis;
    Ref.dbasis = {@d1basis,@d2basis};
    % add internal integration nodes
    Gp = 2;
    [X,Y,Wx,Wy]=triquad(Gp,[0,0 ; 1,0 ; 0,1]);
    Ref.intx = [X(:) Y(:)];
    w = Wx*(Wy');
    Ref.intw = w(:);
    ord = length(Ref.intw);
    % evaluate basis functions at internal integration nodes
    Ref.baseval = zeros(ord,Ref.s);
    Ref.dbaseval = zeros(2,ord,Ref.s);
    for i=1:N
        Ref.baseval(:,i)=basis(i,Ref.intx);
        Ref.dbaseval(1,:,i)=d1basis(i,Ref.intx);
        Ref.dbaseval(2,:,i)=d2basis(i,Ref.intx);
    end
    % add boundary integration nodes
    [x,w] = lgwt(Gp,0,1);
    [X,I] = sort(x);
    W = w(I);
    Ref.bdrx = {X*[1,0] , X*[0,1]+(1-X)*[1,0] , (1-X)*[0,1]};
    Ref.bdrw = {W , W, W};
    % evaluate basis functions at boundary integration nodes
    Ref.basevalbdr = cell(3,1);
    Ref.dbasevalbdr = cell(3,1);
    for k = 1:3
        ord = length(Ref.bdrw{k});
        Ref.basevalbdr{k} = zeros(ord,Ref.s);
        Ref.dbasevalbdr{k} = zeros(2,ord,Ref.s);
        for i=1:N
            Ref.basevalbdr{k}(:,i)=basis(i,Ref.bdrx{k});
            Ref.dbasevalbdr{k}(1,:,i)=d1basis(i,Ref.bdrx{k});
            Ref.dbasevalbdr{k}(2,:,i)=d2basis(i,Ref.bdrx{k});
        end
    end
    
    rFEM = FEM;
    rFEM.Ref = Ref;
    info_end(FEM);
end
