function rFEM = assembleLS(FEM)

fprintf('assembling ... ');
tic;

p = FEM.parameters;
Vh = FEM.Vh;

% problem specific ...
Aref = Laplace_element_stiffness_matrix(FEM);
b = zeros(Vh.nnodes,1);

% integration nodes and weights
one = ones(size(FEM.Ref.intx,1),1);
intx = [one , FEM.Ref.intx];
% loop over elements
I=zeros(Vh.M*FEM.Ref.s^2,1);
J=zeros(Vh.M*FEM.Ref.s^2,1);
U=zeros(Vh.M*FEM.Ref.s^2,1);
cnt = 0;
for k = 1:Vh.M
    % current element
    e = FEM.Th.elements(k,:);
    % compute affine map
    F = FEM.Ref.makeF*FEM.Th.corners(e,:);
    % derivative
    dF = F(2:3,:);
    % corresponding Jacobi determinant
    Jac = det(dF);
    if Jac<=0
        error('element %d has wrong orientation',k);
    end
    % inverse
    dFi = inv(dF);
    % and weight matrix
    W = Jac*dFi'*dFi;
    W = W(:)';
    % compute rhs weights and points
    x = intx*F;
    fdx = Jac*FEM.Ref.intw.*p.f(x);
    % distribute contributions
    for i = 1:FEM.Ref.s
        nd = Vh.elementnodes(k,i);
        b(nd) = b(nd) + fdx'*FEM.Ref.baseval(:,i);
        for j = 1:FEM.Ref.s
            nd2 = Vh.elementnodes(k,j);
                cnt = cnt+1;
                I(cnt) = nd;
                J(cnt) = nd2;
                U(cnt) = W*Aref{i,j}(:);
         end
    end
end
matrices.Afull = sparse(I(1:cnt),J(1:cnt),U(1:cnt),Vh.nnodes,Vh.nnodes);
% add inhomogeneous Dirichlet condition
G = zeros(Vh.nnodes,1);
G(Vh.Dirinodes) = p.g(Vh.nodes(Vh.Dirinodes,:));
matrices.bfull = b-matrices.Afull*G;

% restrict to subspace 
matrices.b = matrices.bfull(Vh.doftonode);
matrices.A = matrices.Afull(Vh.doftonode,Vh.doftonode);
matrices.G = G;

rFEM = FEM;
rFEM.matrices = matrices;

fprintf('%.2fs\n',toc);
