function rFEM = postproc(FEM)

info_start(FEM,'postprocessing');

rFEM = FEM;
% set up full solution if there are Dirichlet nodes
if ~isempty(FEM.Th.Dirichlet)
    rFEM.solution.ufull = FEM.matrices.G;
    rFEM.solution.ufull(FEM.Vh.doftonode) = FEM.solution.u;
else
    rFEM.solution.ufull = FEM.solution.u;
end
% compute gradient of solution
Ref = FEM.Ref;
% compute centroid of reference element
cent = mean(Ref.nodes(Ref.shape,:),1);
% evaluate gradient of basis functions at centroid
dim = FEM.Th.dim;
comb = zeros(dim,Ref.s);
for k = 1:length(Ref.dbasis)
  for i = 1:Ref.s
      comb(k,i) = Ref.dbasis{k}(i,cent);
  end
end
Vh = FEM.Vh;
% prepare result
P = zeros(Vh.M,dim);
G = zeros(Vh.M,dim);
% prepare patch information if possible
makesubpatch = isfield(FEM.Ref,'subpatch');
if makesubpatch
    nrefpatch = size(FEM.Ref.subpatch,1);
    q = size(FEM.Ref.subpatch,2);
    finepatch = zeros(nrefpatch*Vh.M,q);
    cnt = 0;
end
% loop over elements
for k = 1:Vh.M
  % current element
  e = FEM.Th.elements(k,:);
  % compute affine map
  F = FEM.Ref.makeF*FEM.Th.corners(e,:);
  % derivative
  dF = F(2:3,:);
  % vertex nodes of the element
  nds = Vh.elementnodes(k,Ref.shape);
  % compute centroid
  P(k,:) = mean(Vh.nodes(nds,:),1);
  % relevant solution components
  vrel = rFEM.solution.ufull(Vh.elementnodes(k,:));
  G(k,:) = dF\(comb*vrel);
  % sub elements for plotting the solution
  if makesubpatch
      for l = 1:nrefpatch
         finepatch(cnt+l,:) = Vh.elementnodes(k,FEM.Ref.subpatch(l,:));
      end
      cnt = cnt + nrefpatch;
  end
end
rFEM.solution.gradient.P = P;
rFEM.solution.gradient.G = G;
if makesubpatch
  rFEM.solution.finepatch = finepatch;
end
info_end(FEM);
