function rFEM = trimesh(FEM)
% construct regular triangular grid in rectangle

info_start(FEM,'triangulation');

mi = FEM.parameters;
% dimension of the domain
Th.dim = 2;
% get grid information
Nx = mi.Nx;
Ny = mi.Ny;

% index fields to address points
ix = 0:Nx;
iy = 0:Ny;
% two dimensional index fields
[IX,IY] = meshgrid(ix,iy);
% indices of lower left corners of squares
SW = find(IX<ix(end) & IY<iy(end));
% left upper corners
NW = find(IX<ix(end) & IY>iy(1));
% lower right
SE = find(IX>ix(1) & IY<iy(end));
% upper right
NE = find(IX>ix(1) & IY>iy(1));
% set up triangulation information
% points in counterclockwise order
Th.elements = [SW SE NW;
               NE NW SE];
% number of elements
Th.M = size(Th.elements,1);

% spacing
dxs = 1/ix(end)*(mi.X(2)-mi.X(1));
dys = 1/iy(end)*(mi.Y(2)-mi.Y(1));
% coordinates in scaled geometry
Th.corners = [dxs*IX(:)+mi.X(1), dys*IY(:)+mi.Y(1)];
Th.nc = size(Th.corners,1);

% find segments of Dirichlet boundary 
% top
Th.Dirichlet = [find(IY==iy(end) & IX<ix(end)) , find(IY==iy(end) & IX>ix(1))];
% right
Th.Dirichlet = [Th.Dirichlet ; find(IX==ix(end) & IY<iy(end)) , find(IX==ix(end) & IY>iy(1))];
% bottom
Th.Dirichlet = [Th.Dirichlet ; find(IY==iy(1) & IX<ix(end)) , find(IY==iy(1) & IX>ix(1))];
% left
Th.Dirichlet = [Th.Dirichlet ; find(IX==ix(1) & IY<iy(end)) , find(IX==ix(1) & IY>iy(1))];

rFEM = FEM;
rFEM.Th = Th;

info_end(FEM);
